<?php

namespace Drupal\simple_favs\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\node\Entity\Node;

/**
 * Provides a "My Favourites" block for Simple Favs.
 *
 * @Block(
 *   id = "simple_favs_block",
 *   admin_label = @Translation("Simple Favs"),
 * )
 */
class SimpleFavsBlock extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function build() {
    // Get the interface language ID.
    $langcode = \Drupal::languageManager()->getCurrentLanguage()->getId();

    $config = \Drupal::config('simple_favs.settings');
    // Load language-specific config override.
    $config_lang = \Drupal::languageManager()->getLanguageConfigOverride($langcode, 'simple_favs.settings');

    $wrapper_prefix = $config_lang->get('wrapper_prefix') ?? '';
    $wrapper_suffix = $config_lang->get('wrapper_suffix') ?? '';
    $title_prefix = $config_lang->get('title_prefix') ?? '';
    $title_suffix = $config_lang->get('title_suffix') ?? '';
    $fallback_prefix = $config_lang->get('fallback_prefix') ?? '';
    $fallback_suffix = $config_lang->get('fallback_suffix') ?? '';
    $title_classes = $config_lang->get('title_classes') ?? '';
    $favs_list_classes = $config_lang->get('favs_list_classes') ?? '';

    // Safe fallback for language (supports single-language sites).
    $langcode = 'und';
    if (\Drupal::hasService('language_manager')) {
      $langcode = \Drupal::languageManager()->getCurrentLanguage()->getId();
    }

    $uid = \Drupal::currentUser()->id();
    $use_database = $config->get('use_database_storage') ?? FALSE;

    $favs = [];
    $favs_other = [];
    $list_markup = '';

    if ($uid && $use_database) {
      $connection = \Drupal::database();
      $result = $connection->select('simple_favs_user', 's')
        ->fields('s', ['favs'])
        ->condition('uid', $uid)
        ->execute()
        ->fetchField();

      if ($result) {
        $favs = unserialize($result);
      }

      $query = $connection->select('simple_favs_user_other', 'o')
        ->fields('o', ['path', 'title'])
        ->condition('uid', $uid)
        ->execute();

      foreach ($query as $record) {
        $favs_other[] = ['path' => $record->path, 'title' => $record->title];
      }
    }

    if (!empty($favs) || !empty($favs_other)) {
      $items = [];

      foreach ($favs as $nid) {
        $node = Node::load($nid);
        if ($node && $node->access('view')) {
          $url = '/node/' . $nid;
          if ($node->hasTranslation($langcode)) {
            $node = $node->getTranslation($langcode);
            $url = '/' . $langcode . '/node/' . $nid;
          }
          $title = $node->label();
          $items[] = '<li><a href="' . $url . '">' . $title . '</a></li>';
        }
      }

      foreach ($favs_other as $fav) {
        $path = $fav['path'] ?? '';
        $title = $fav['title'] ?? '';
        if (!empty($path) && !empty($title)) {
          $items[] = '<li><a href="' . $path . '">' . $title . '</a></li>';
        }
      }

      if (!empty($items)) {
        $list_markup = '<ul class="simple-favs-list ' . $favs_list_classes . '">' . implode('', $items) . '</ul>';
      }
    }

    if (empty($list_markup)) {
      $list_markup = '<p class="' . $favs_list_classes . '">' . $this->t('There currently are no selected favourites.') . '</p>';
    }

    return [
      '#type' => 'container',
      '#attributes' => [
        'data-block-plugin-id' => 'simple-favs-block',
        'class' => ['simple-favs-wrapper'],
      ],
      'prefix' => ['#markup' => $wrapper_prefix],
      'title' => [
        '#markup' => $title_prefix . '<h2 class="simple-favs ' . $title_classes . '">' . $this->t('My Favourites') . '</h2>' . $title_suffix,
      ],
      'favs' => [
        '#markup' => $fallback_prefix . $list_markup . $fallback_suffix,
      ],
      'suffix' => ['#markup' => $wrapper_suffix],
      '#cache' => [
        'contexts' => $this->getCacheContexts(),
        'tags' => $this->getCacheTags(),
      ],
      '#attached' => [
        'library' => ['simple_favs/favs'],
        'drupalSettings' => [
          'simple_favs' => [
            'block' => [
              'title_prefix' => $title_prefix,
              'title_suffix' => $title_suffix,
              'fallback_prefix' => $fallback_prefix,
              'fallback_suffix' => $fallback_suffix,
              'wrapper_prefix' => $wrapper_prefix,
              'wrapper_suffix' => $wrapper_suffix,
              'title_classes' => $title_classes,
              'favs_list_classes' => $favs_list_classes,
              'langcode' => $langcode,
            ],
          ],
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    $tags = ['config:simple_favs.settings', 'simple_favs:block'];
    $uid = \Drupal::currentUser()->id();

    if ($uid) {
      $tags[] = 'simple_favs:user:' . $uid;
    }

    return $tags;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    return ['user.roles', 'languages:language_interface'];
  }

}
